/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.search.fieldcollapse;

/**
 * A <code>CollapseGroup</code> represents a logical group where documents are collaped into.
 * Per {@link DocumentCollapser} the definition of a collapse group may differ.
 * <p>
 * The uniqueness of the group is specified by its hashcode and equals method. All {@link DocumentCollapser} implementations
 * must have a proper implementation of these methods.
 */
public interface CollapseGroup {

  /**
   * Returns the value for presentation purposes in the response.
   * This may not be an unique value.
   *
   * @return the value for presentation purposes in the response
   */
  String getKey();

}
