/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.solr.search.fieldcollapse;

import org.apache.solr.search.DocSet;

/**
 * Contains the collapsed and uncollapsed result.
 */
public class DocumentCollapseResult {

    private final DocSet collapsedDocset;
    private final DocSet unCollapsedDocset;

    /**
     * Constructs a <code>DocumentCollapseResult</code>.
     *
     * @param collapsedDocset The collapsed docset
     * @param unCollapsedDocset The uncollapsed docset
     */
    public DocumentCollapseResult(DocSet collapsedDocset, DocSet unCollapsedDocset) {
        this.collapsedDocset = collapsedDocset;
        this.unCollapsedDocset = unCollapsedDocset;
    }

    /**
     * Returns the collapsed docset.
     *
     * @return the collapsed docset
     */
    public DocSet getCollapsedDocset() {
        return collapsedDocset;
    }

    /**
     * Returns the uncollapsed docset.
     *
     * @return the uncollapsed docset
     */
    public DocSet getUnCollapsedDocset() {
        return unCollapsedDocset;
    }
}
