/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.search.fieldcollapse.collector;

import org.apache.solr.common.util.NamedList;
import org.apache.solr.search.DocList;
import org.apache.solr.search.fieldcollapse.CollapseGroup;

/**
 * A <code>CollapseCollector</code> is responsible for receiving collapse callbacks from the <code>DocumentCollapser</code>.
 * An implementation can choose what to do with the received callbacks and data. Whatever an implementation collects it
 * is responsible for adding its results to the response.
 *
 * Implementation of this interface don't need to be thread safe!
 */
public interface CollapseCollector {

  /**
   * Informs the <code>CollapseCollector</code> that a document has been collapsed under the specified collapseGroup.
   *
   * @param docId The id of the document that has been collasped
   * @param collapseGroup The collapse group the docId has been collapsed under
   * @param collapseContext The collapse context
   */
  void documentCollapsed(int docId, CollapseGroup collapseGroup, CollapseContext collapseContext);

  /**
   * Informs the <code>CollapseCollector</code> about the document head.
   * The document head is the most relevant id for the specified collapseGroup.
   *
   * @param docHeadId The identifier of the document head
   * @param collapseGroup The collapse group of the document head
   * @param collapseContext The collapse context
   */
  void documentHead(int docHeadId, CollapseGroup collapseGroup, CollapseContext collapseContext);

  /**
   * Adds the <code>CollapseCollector</code> implementation specific result data to the result.
   *
   * @param result The response result 
   * @param docs The documents to be added to the response
   * @param collapseContext The collapse context
   */
  void getResult(NamedList result, DocList docs, CollapseContext collapseContext);

}
