/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.search.fieldcollapse.collector;

import org.apache.solr.request.SolrQueryRequest;

/**
 * A concrete <code>CollapseCollectorFactory</code> implementation is responsible for creating {@link CollapseCollector}
 * instances based on the {@link SolrQueryRequest}.
 */
public interface CollapseCollectorFactory {

  /**
   * Creates an instance of a CollapseCollector specified by the concrete subclass.
   * The concrete subclass decides based on the specified request if an new instance has to be created and
   * can return <code>null</code> for that matter.
   * 
   * @param request The specified request
   * @return an instance of a CollapseCollector or <code>null</code>
   */
  CollapseCollector createCollapseCollector(SolrQueryRequest request);

}
