/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.search.fieldcollapse.collector.aggregate;

import org.apache.solr.search.fieldcollapse.CollapseGroup;

/**
 * A concrete aggregate function implementation is responsible for calculating the aggregated information and is used
 * by {@link org.apache.solr.search.fieldcollapse.collector.AggregateCollapseCollectorFactory}
 * and its <code>CollapseCollector</code> to provide the aggregated data.
 */
public interface AggregateFunction {

  /**
   * Adds the specified number that a list of other numbers that are associated with the specified field (group)
   *
   * @param collapseGroup The field value (group) to add the number to.
   * @param number The number to add
   */
  void collect(CollapseGroup collapseGroup, String number);

  /**
   * Returns the string representation of the calculated aggregate for the specified collapseGroup.
   *
   * @param collapseGroup The collapse group to calculate the aggragate data for.
   * @return the calculated aggregates per field value (group)
   */
  String calculate(CollapseGroup collapseGroup);

  /**
   * Returns the name of the function.
   *
   * @return the name of the function
   */
  String getName();

}
