/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.search.fieldcollapse.collector.aggregate;

import org.apache.solr.search.fieldcollapse.CollapseGroup;
import org.apache.solr.search.fieldcollapse.util.Counter;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Map;

/**
 * Computes the average (Arithmetic mean) of per collapse group.
 */
public class AverageFunction implements AggregateFunction {

  private final Map<CollapseGroup, BigDecimal> sums = new HashMap<CollapseGroup, BigDecimal>();
  private final Map<CollapseGroup, Counter> times = new HashMap<CollapseGroup, Counter>();

  public void collect(CollapseGroup collapseGroup, String value) {
    BigDecimal number = sums.get(collapseGroup);
    if (number == null) {
      sums.put(collapseGroup, new BigDecimal(value));
      times.put(collapseGroup, new Counter(1));
    } else {
      number = number.add(new BigDecimal(value));
      sums.put(collapseGroup, number);
      times.get(collapseGroup).increment();
    }
  }

  public String calculate(CollapseGroup collapseGroup) {
    BigDecimal number = sums.get(collapseGroup);
    if (number == null) {
      return null;
    }

    int nrOfOccurences = times.get(collapseGroup).getCount();
    // TODO: need to find a way to specify the scale via the request
    BigDecimal result = number.divide(new BigDecimal(nrOfOccurences), 2, RoundingMode.HALF_UP);
    return result.toString();
  }

  public String getName() {
    return "avg";
  }

}
