/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.search.fieldcollapse.util;

/**
 * A simple counter. In some parts of the field collapse code I need to keep track of the count. If I use an Integer for
 * that I have the contant overhead of autoboxing and the creation of new Integer objects when I increment a counter.
 * This class does not have that overhead because I increment the same variable.
 * <p>
 * This class is <b>not</b> thread safe.
 */
public class Counter {

  private int count;

  /**
   * Constructs a counter with count value zero.
   */
  public Counter() {
    this(0);
  }

  /**
   * Constructs a counter with the specified count.
   *
   * @param count The specified count
   */
  public Counter(int count) {
    this.count = count;
  }

  /**
   * Increments the counter.
   */
  public void increment() {
    count++;
  }

  /**
   * Sets the counter to the specified count
   *
   * @param count The specified count
   */
  public void setCount(int count) {
    this.count = count;
  }

  /**
   * Returns the current count of this counter.
   *
   * @return the current count
   */
  public int getCount() {
    return count;
  }

}
